#pragma once

#include "compiler.h"

#ifdef __cplusplus
#include <cstdint>
#else
#include <stdint.h>
#endif // __cplusplus

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus
#include <openssl/ec.h>
#include <openssl/ecdsa.h>
#ifdef __cplusplus
}
#endif // __cplusplus

typedef enum
{
	Unspecified = 0,
	PrimeShortWeierstrass = 1,
	PrimeTwistedEdwards = 2,
	PrimeMontgomery = 3,
	Characteristic2 = 4
} ECCurveType;

/*
Returns the new EC_KEY instance using the explicit parameters.
*/
DLLEXPORT EC_KEY* CryptoNative_EcKeyCreateByExplicitParameters(
	ECCurveType curveType,
	uint8_t* qx, int32_t qxLength,
	uint8_t* qy, int32_t qyLength,
	uint8_t* d, int32_t dLength,
	uint8_t* p, int32_t pLength,
	uint8_t* a, int32_t aLength,
	uint8_t* b, int32_t bLength,
	uint8_t* gx, int32_t gxLength,
	uint8_t* gy, int32_t gyLength,
	uint8_t* order, int32_t nLength,
	uint8_t* cofactor, int32_t hLength,
	uint8_t* seed, int32_t sLength);

/*
Shims the ECDSA_sign method.
Returns 1 on success, otherwise 0.
*/
DLLEXPORT int32_t
CryptoNative_EcDsaSignEx(const uint8_t* dgst, int32_t dgstlen, uint8_t* sig, int32_t* siglen, uint8_t* kinv, int32_t kinvLength, uint8_t* rp, int32_t rpLength, EC_KEY* key);


/*
Shims the ECDSA_size method.
Returns the maximum length of a DER encoded ECDSA signature created with this key.
*/
DLLEXPORT int32_t CryptoNative_EcDsaSize(const EC_KEY* key);

DLLEXPORT int32_t
CryptoNative_EcPspParameter(const EC_KEY* key, const uint8_t* sha256, int32_t sha256len, BIGNUM** kinv, int32_t* kinvlen, BIGNUM** rp, int32_t* rplen);
